@@ -1,7 +1,7 @@
 ##
 # name:      Moos
 # abstract:  Moo s{imple,peedy,ingle}
-# author:    Ingy döt Net <ingy@cpan.org>
+# author: Ingy döt Net <ingy@cpan.org>
 # license:   perl
 # copyright: 2012
 # see:
@@ -22,8 +22,9 @@ package Moos;
 use v5.10.0;
 use mro;
 use Scalar::Util;
+use Carp qw(confess);
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 sub import {
     # Get name of the "class" from whence "use Moos;"
@@ -43,6 +44,10 @@ sub import {
     _export($package, has => \&has, $meta);
     _export($package, extends => \&extends, $meta);
 
+    # Export the 'blessed' and 'confess' functions
+    _export($package, blessed => \&Scalar::Util::blessed);
+    _export($package, confess => \&Carp::confess);
+
     # Possibly export some handy debugging stuff
     _export_xxx($package) if $ENV{PERL_MOOS_XXX};
 }
@@ -52,11 +57,11 @@ sub has {
     my ($meta, $name) = splice(@_, 0, 2);
     my %args;
 
-    # Support 2-arg shorthand: 
+    # Support 2-arg shorthand:
     #     has foo => 42;
     if (@_ % 2) {
         my $default = shift;
-        my $sub = 
+        my $sub =
             ref($default) eq 'HASH' ? sub {+{%$default}} :
             ref($default) eq 'ARRAY' ? sub {[@$default]} :
             sub {$default};
@@ -69,6 +74,8 @@ sub has {
 
     # Make a Setter/Getter accessor
     my ($builder, $default) = @args{qw(builder default)};
+    $builder = "_build_$name"
+        if defined $builder && $builder eq "1";
     my $accessor =
         $builder ? sub {
             $#_ ? $_[0]{$name} = $_[1] :
@@ -84,12 +91,53 @@ sub has {
             $#_ ? $_[0]{$name} = $_[1] : $_[0]{$name};
         };
 
+    # Support (is => 'ro')
+    if (exists $args{is} and $args{is} eq 'ro') {
+        my $orig = $accessor;
+        $accessor = sub {
+            confess "cannot set value for read-only accessor '$name'"
+                if @_ > 1;
+            goto $orig;
+        };
+    }
+
     # Dev debug thing to trace calls to accessor subs.
     $accessor = _trace_accessor_calls($name, $accessor)
         if $ENV{PERL_MOOS_ACCESSOR_CALLS};
 
     # Export the accessor.
     _export($meta->{package}, $name, $accessor);
+
+    # Clearer
+    if (exists $args{clearer}) {
+        my $clearer = $args{clearer};
+        $clearer = $name =~ /^_/ ? "_clear$name" : "clear_$name"
+            if $clearer eq "1";
+        my $sub = sub { delete $_[0]{$name} };
+        _export($meta->{package}, $clearer, $sub);
+    }
+
+    # Predicate
+    if (exists $args{predicate}) {
+        my $predicate = $args{predicate};
+        $predicate = $name =~ /^_/ ? "_has$name" : "has_$name"
+            if $predicate eq "1";
+        my $sub = sub { exists $_[0]{$name} };
+        _export($meta->{package}, $predicate, $sub);
+    }
+
+    # Delegated methods
+    if (exists $args{handles}) {
+        my %map;
+        %map = %{$args{handles}}
+            if Scalar::Util::reftype($args{handles}) eq 'HASH';
+        %map = map { ;$_=>$_ } @{$args{handles}}
+            if Scalar::Util::reftype($args{handles}) eq 'ARRAY';
+        while (my ($local, $remote) = each %map) {
+            my $sub = sub { shift->{$name}->$remote(@_) };
+            _export($meta->{package}, $local, $sub);
+        }
+    }
 }
 
 # Inheritance maker
@@ -148,18 +196,27 @@ sub _trace_accessor_calls {
 # The remainder of this module was heavily inspired by Moose, and tried to do
 # what Moose does, only much less.
 package Moos::Meta::Class;
+use Carp qw(confess);
 
+# Store all the Moos meta-class-objects in a private hash, keyed on
+# package/class name:
 my $meta_class_objects = {};
 
+# Helper method to get class name:
 sub name { $_[0]->{package} }
 
+# Either looking the existing meta-class-object or register a new one:
 sub initialize {
     my ($class, $package) = @_;
 
+    # This is a tiny version of a Moose meta-class-object.
+    # We really just need a place to keep the attributes.
     return $meta_class_objects->{$package} //= do {
         bless {
             package => $package,
+            # This isn't currently used but matches Moose and is cheap.
             attributes => {},
+            # We construct with attribute in order defined. (Unlike Moose)
             _attributes => [],
         }, $class;
     };
@@ -179,6 +236,7 @@ sub add_attribute {
     );
 }
 
+# This is where new objects are constructed. (Moose style)
 sub new_object {
     my ($self, $params) = @_;
     my $object = $self->_construct_instance($params);
@@ -198,17 +256,24 @@ sub _construct_instance {
         }
         if (not $attr->{lazy}) {
             if (my $builder = $attr->{builder}) {
+                $builder = "_build_$name"
+                    if defined $builder && $builder eq "1";
                 $instance->{$name} = $instance->$builder();
                 next;
             }
             elsif (my $default = $attr->{default}) {
                 $instance->{$name} = $default->($instance);
             }
+            if ($attr->{required} and not exists $instance->{$name}) {
+                confess "missing required attribute '$name'";
+            }
         }
     }
     return $instance;
 }
 
+# Return all the unique attributes in the order defined from the outer class
+# inwards:
 sub get_all_attributes {
     my $self = shift;
     my (@attrs, %attrs);
@@ -223,8 +288,10 @@ sub get_all_attributes {
     return @attrs;
 }
 
+# This is the default base class for all Moos classes:
 package Moos::Object;
 
+# Moos constructor
 sub new {
     my $class = shift;
     my $real_class = Scalar::Util::blessed($class) || $class;
@@ -232,10 +299,12 @@ sub new {
     return Moos::Meta::Class->initialize($real_class)->new_object($params);
 }
 
+# A default BUILDARGS
 sub BUILDARGS {
     return {@_[1..$#_]};
 }
 
+# A default BUILDALL
 sub BUILDALL {
     return unless $_[0]->can('BUILD');
     my ($self, $params) = @_;
@@ -249,6 +318,7 @@ sub BUILDALL {
     }
 }
 
+# A Data::Dumper method. (Moose has it. No cost.)
 sub dump {
     no warnings 'once';
     my $self = shift;
@@ -257,6 +327,8 @@ sub dump {
     Data::Dumper::Dumper $self;
 }
 
+# Use to retrieve the (rather useless) Moos meta-class-object. Hey it's a
+# start. :)
 sub meta {
     Moos::MOP::Class->initialize(Scalar::Util::blessed($_[0]) || $_[0]);
 }
@@ -271,7 +343,7 @@ sub meta {
     extends 'Boos';
 
     has this => ();
-    has that => (default => sub { 42 });
+    has that => 42;
     has other => (
         builder => 'build_other',
         lazy => 1,
@@ -282,13 +354,18 @@ sub meta {
         # build, build, build
     }
 
+    sub BUILDARGS {
+        my ($self, @args) = @_;
+        # munge, munge, munge
+        return {%munged_args};
+    }
+
 =head1 DESCRIPTION
 
 Moos completes the M to Moose sequence of Perl OO modules.
 
 This one is pure Perl, no dependencies, single file and Moose compatible (for
-what it does). It is fairly minimal; it supports the features shown in the
-L<SYNOPSIS>.
+what it does). It is fairly minimal.
 
 =head1 FEATURES
 
@@ -297,6 +374,10 @@ L<Moos>:
 
 =over
 
+=item strict / warnings
+
+Turns on C<strict> and C<warnings> for you.
+
 =item extends
 
 For inheritance. C<Moos::Object> is the default base class.
@@ -318,13 +399,25 @@ return is ignored.
 
     sub BUILD { my $self = shift; ... }
 
+=item Helpful exports
+
+The ever useful C<blessed> (from L<Scalar::Util>) and C<confess> (from
+L<Carp>) are exported to your namespace.
+
 =item has
 
-Accessor generator. Supports the C<default>, C<build> and C<lazy> options,
-described below.
+Accessor generator. Supports the C<is>, C<default>, C<build>, C<lazy>,
+C<clearer>, C<predicate>, C<required> and C<handles> options, described below.
 
     has this => ();
 
+=item is
+
+Specify which type of attribute accessor to provide. The default is "rw",
+a read-write accessor. Read-only "ro" accessors are also supported.
+
+    has this => ( is => "ro" );
+
 =item default
 
 Specify the sub to generate a default value.
@@ -335,32 +428,64 @@ Specify the sub to generate a default value.
 
 Specify the method name to generate a default value.
 
-    has this => ( builder => 'build_this' );
+    has this => ( builder => '_build_this' );
+    has that => ( builder => 1 );  # accept default name for method
 
 =item lazy
 
 Don't generate defaults during object construction.
 
-    has this => ( builder => 'build_this', lazy => 1 );
+    has this => ( builder => '_build_this', lazy => 1 );
+
+=item clearer
+
+Creates a clearer method.
+
+    has this => ( clearer => "clear_this" );
+    has that => ( clearer => 1 );  # accept default name for method
+
+=item predicate
+
+Creates a predicate method, which can be used to check if the attribute is
+set or unset.
+
+    has this => ( predicate => "has_this" );
+    has that => ( predicate => 1 );  # accept default name for method
+
+=item required
+
+Require that a value for the attribute be provided to the constructor or
+generated during object construction.
+
+    has this => ( required => 1 );
+
+=item handles
+
+Delegated method calls.
+
+    has wheels => (handles => [qw/ roll /]);
+
+This accepts a hashref or arrayref, but not the other possibilities
+offered by L<Moose>.
 
 =back
 
-Note that currently all accessors are read-write, and the C<is> keyword is
-silently ignored (as are all other unknown keywords).
+Note that currently all accessors are read-write by default and all unknown
+options are silently ignored.
 
 =head1 HAS DIFFERENCES
 
 Moos has a few differences from Moose, regarding it's accessor support (ie the
 'has' function).
 
-Support for C<default>, C<builder> and C<lazy> are about the same. All other
-arguments are currently ignored, including the C<is> property. All generated
-accessors are 'rw'. So you can just say:
+The supported options detailed above are about the same as Moose. All other
+arguments are currently ignored. All generated accessors are 'rw'. So you can
+just say:
 
     has 'this';
     has that => ();
 
-Unlike the other Mo* modules, Moos also supports just sepcifiying the default.
+Unlike the other Mo* modules, Moos also supports just specifying the default.
 If the number of arguments (after the name) is an odd number, then the first
 value is the default. The following forms are valid:
 
@@ -406,6 +531,8 @@ this broke my toolchain (TestML, Module::Install, etc).
 I tried to inline L<Moo> into one file but failed, and ended up with this.
 I've shared Pegex::Base as L<Moos> in case any other projects want it.
 
+Later on, Toby added a bunch of low-cost but very features from Moose.
+
 =head1 ON SPEED
 
 In the end, I got Pegex to run even faster with Moos than it originally did
@@ -438,3 +565,8 @@ calls like they would expect to.
 
 I'm sure I've missed some subtlties, and would be glad to hear opinions, but
 in the meantime I'm happy that my code is faster and pure Perl.
+
+=head1 CREDITS
+
+Toby Inkster <tobyink@cpan.org> submitted a number of great pull requests,
+making Moos a little bit more like Moose.
